Package: DBmaps
Title: A Metadata-Driven Framework for Streamlining Database Joins
Version: 0.1.0
Authors@R: 
    c(
        person("Akshat", "Maurya", ,"codingmaster902@gmail.com", role = c("aut", "cre")),
        person("David", "Shilane", ,"david.shilane@columbia.edu", role = c("aut"))
    )
Description: 
    Simplifies and automates the process of exploring and merging 
    data from relational databases. This package allows users to discover 
    table relationships, create a map of all possible joins, and generate 
    executable plans to merge data based on a structured metadata framework.
URL: https://github.com/akshat09867/DBmaps
BugReports: https://github.com/akshat09867/DBmaps/issues
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), DiagrammeR
Config/testthat/edition: 3
VignetteBuilder: knitr
Imports: data.table
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2025-09-03 06:25:17 UTC; akshat
Author: Akshat Maurya [aut, cre],
  David Shilane [aut]
Maintainer: Akshat Maurya <codingmaster902@gmail.com>
Repository: CRAN
Date/Publication: 2025-09-08 19:20:01 UTC
Built: R 4.6.0; ; 2026-04-25 18:07:04 UTC; unix
