# Generated by roxygen2: do not edit by hand

S3method(add_table,MetadataRegistry)
export(add_table)
export(create_join_plan)
export(create_metadata_registry)
export(execute_join_plan)
export(generate_aggregation_code)
export(map_join_paths)
export(plot_join_plan)
export(table_info)
importFrom(data.table,as.data.table)
importFrom(data.table,data.table)
importFrom(data.table,is.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setcolorder)
